From abcb146f0c4e5c99e5e44349a65e6fd49e9296d9 Mon Sep 17 00:00:00 2001
From: James Moger <james.moger@gitblit.com>
Date: Tue, 30 Oct 2012 17:03:02 -0400
Subject: [PATCH] Moved owner field onto access permissions tab

---
 src/com/gitblit/models/UserModel.java |  133 +++++++++++++++++++++++++++++++++++++++----
 1 files changed, 119 insertions(+), 14 deletions(-)

diff --git a/src/com/gitblit/models/UserModel.java b/src/com/gitblit/models/UserModel.java
index 6fe8df2..22f250c 100644
--- a/src/com/gitblit/models/UserModel.java
+++ b/src/com/gitblit/models/UserModel.java
@@ -17,14 +17,19 @@
 
 import java.io.Serializable;
 import java.security.Principal;
-import java.util.HashMap;
+import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashSet;
+import java.util.LinkedHashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
 import com.gitblit.Constants.AccessPermission;
 import com.gitblit.Constants.AccessRestrictionType;
 import com.gitblit.Constants.AuthorizationControl;
+import com.gitblit.Constants.PermissionType;
+import com.gitblit.Constants.RegistrantType;
 import com.gitblit.Constants.Unused;
 import com.gitblit.utils.ArrayUtils;
 import com.gitblit.utils.StringUtils;
@@ -56,7 +61,7 @@
 	// retained for backwards-compatibility with RPC clients
 	@Deprecated
 	public final Set<String> repositories = new HashSet<String>();
-	public final Map<String, AccessPermission> permissions = new HashMap<String, AccessPermission>();
+	public final Map<String, AccessPermission> permissions = new LinkedHashMap<String, AccessPermission>();
 	public final Set<TeamModel> teams = new HashSet<TeamModel>();
 
 	// non-persisted fields
@@ -125,6 +130,31 @@
 	}
 	
 	/**
+	 * Returns a list of repository permissions for this user exclusive of
+	 * permissions inherited from team memberships.
+	 * 
+	 * @return the user's list of permissions
+	 */
+	public List<RegistrantAccessPermission> getRepositoryPermissions() {
+		List<RegistrantAccessPermission> list = new ArrayList<RegistrantAccessPermission>();
+		for (Map.Entry<String, AccessPermission> entry : permissions.entrySet()) {
+			String registrant = entry.getKey();
+			boolean editable = true;
+			PermissionType pType = PermissionType.EXPLICIT;
+			if (isMyPersonalRepository(registrant)) {
+				pType = PermissionType.OWNER;
+				editable = false;
+			} else if (StringUtils.findInvalidCharacter(registrant) != null) {
+				// a regex will have at least 1 invalid character
+				pType = PermissionType.REGEX;
+			}
+			list.add(new RegistrantAccessPermission(registrant, entry.getValue(), pType, RegistrantType.REPOSITORY, editable));
+		}
+		Collections.sort(list);
+		return list;
+	}
+	
+	/**
 	 * Returns true if the user has any type of specified access permission for
 	 * this repository.
 	 * 
@@ -148,6 +178,18 @@
 			}
 		}
 		return false;
+	}
+	
+	/**
+	 * Returns true if the user has an explicitly specified access permission for
+	 * this repository.
+	 * 
+	 * @param name
+	 * @return if the user has an explicitly specified access permission
+	 */
+	public boolean hasExplicitRepositoryPermission(String name) {
+		String repository = AccessPermission.repositoryFromRole(name).toLowerCase();
+		return permissions.containsKey(repository);
 	}
 	
 	/**
@@ -186,38 +228,42 @@
 			return AccessPermission.REWIND;
 		}
 		
-		// determine best permission available based on user's personal permissions
-		// and the permissions of teams of which the user belongs
+		// explicit user permission OR user regex match is used
+		// if that fails, then the best team permission is used
 		AccessPermission permission = AccessPermission.NONE;
 		if (permissions.containsKey(repository.name.toLowerCase())) {
-			// exact repository permission specified
+			// exact repository permission specified, use it
 			AccessPermission p = permissions.get(repository.name.toLowerCase());
 			if (p != null) {
-				permission = p;
+				return p;
 			}
 		} else {
 			// search for regex permission match
 			for (String key : permissions.keySet()) {
-				if (repository.name.matches(key)) {
+				if (StringUtils.matchesIgnoreCase(repository.name, key)) {
 					AccessPermission p = permissions.get(key);
 					if (p != null) {
+						// take first match
 						permission = p;
+						break;
 					}
 				}
 			}
 		}
 		
-		for (TeamModel team : teams) {
-			AccessPermission p = team.getRepositoryPermission(repository);
-			if (permission == null || p.exceeds(permission)) {
-				// use team permission
-				permission = p;
+		if (AccessPermission.NONE.equals(permission)) {
+			for (TeamModel team : teams) {
+				AccessPermission p = team.getRepositoryPermission(repository);
+				if (p.exceeds(permission)) {
+					// use highest team permission
+					permission = p;
+				}
 			}
 		}
 		return permission;
 	}
 	
-	private boolean canAccess(RepositoryModel repository, AccessRestrictionType ifRestriction, AccessPermission requirePermission) {
+	protected boolean canAccess(RepositoryModel repository, AccessRestrictionType ifRestriction, AccessPermission requirePermission) {
 		if (repository.accessRestriction.atLeast(ifRestriction)) {
 			AccessPermission permission = getRepositoryPermission(repository);
 			return permission.atLeast(requirePermission);
@@ -345,6 +391,27 @@
 		}
 		return false;
 	}
+	
+	/**
+	 * Returns true if the user is allowed to create the specified repository.
+	 * 
+	 * @param repository
+	 * @return true if the user can create the repository
+	 */
+	public boolean canCreate(String repository) {
+		if (canAdmin()) {
+			// admins can create any repository
+			return true;
+		}
+		if (canCreate) {
+			String projectPath = StringUtils.getFirstPathElement(repository);
+			if (!StringUtils.isEmpty(projectPath) && projectPath.equalsIgnoreCase("~" + username)) {
+				// personal repository
+				return true;
+			}
+		}
+		return false;
+	}
 
 	public boolean isTeamMember(String teamname) {
 		for (TeamModel team : teams) {
@@ -405,4 +472,42 @@
 	public int compareTo(UserModel o) {
 		return username.compareTo(o.username);
 	}
+	
+	/**
+	 * Returns true if the name/email pair match this user account.
+	 * 
+	 * @param name
+	 * @param email
+	 * @return true, if the name and email address match this account
+	 */
+	public boolean is(String name, String email) {
+		// at a minimum a usename or display name must be supplied
+		if (StringUtils.isEmpty(name)) {
+			return false;
+		}
+		boolean nameVerified = name.equalsIgnoreCase(username) || name.equalsIgnoreCase(getDisplayName());
+		boolean emailVerified = false;
+		if (StringUtils.isEmpty(emailAddress)) {
+			// user account has not specified an email address
+			// rely on username/displayname verification
+			emailVerified = true;
+		} else {
+			// user account has specified an email address
+			// require email address verification
+			if (!StringUtils.isEmpty(email)) {
+				emailVerified = email.equalsIgnoreCase(emailAddress);
+			}
+		}
+		return nameVerified && emailVerified;
+	}
+	
+	public boolean hasBranchPermission(String repositoryName, String branch) {
+		// Default UserModel doesn't implement branch-level security. Other Realms (i.e. Gerrit) may override this method.
+		return hasRepositoryPermission(repositoryName);
+	}
+	
+	public boolean isMyPersonalRepository(String repository) {
+		String projectPath = StringUtils.getFirstPathElement(repository);
+		return !StringUtils.isEmpty(projectPath) && projectPath.equalsIgnoreCase("~" + username);
+	}
 }
@@ -186,38 +228,42 @@
 			}
 		} else {
-			// search for regex permission match
+			// search for case-insensitive regex permission match
 			for (String key : permissions.keySet()) {

--
Gitblit v1.9.1